@@ -73,6 +73,12 @@
 #include <string.h>
 #include <errno.h>
 
+static const GstElementDetails gst_gnome_vfs_sink_details =
+GST_ELEMENT_DETAILS ("GnomeVFS Sink",
+    "Sink/File",
+    "Write a stream to a GnomeVFS URI",
+    "Bastien Nocera <hadess@hadess.net>");
+
 enum
 {
   SIGNAL_ERASE_ASK,
@@ -139,11 +145,6 @@ static void
 gst_gnome_vfs_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  static GstElementDetails gst_gnome_vfs_sink_details =
-      GST_ELEMENT_DETAILS ("GnomeVFS Sink",
-      "Sink/File",
-      "Write a stream to a GnomeVFS URI",
-      "Bastien Nocera <hadess@hadess.net>");
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sinktemplate));
@@ -241,7 +242,7 @@ gst_gnome_vfs_sink_init (GstGnomeVFSSink * sink, GstGnomeVFSSinkClass * klass)
   sink->uri_name = NULL;
   sink->handle = NULL;
   sink->own_handle = FALSE;
-  sink->data_written = 0;
+  sink->current_pos = 0;
 
   GST_BASE_SINK (sink)->sync = FALSE;
 }
@@ -266,6 +267,8 @@ gst_gnome_vfs_sink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_LOCATION:{
+      const gchar *new_location;
+
       if (sink->uri) {
         gnome_vfs_uri_unref (sink->uri);
         sink->uri = NULL;
@@ -274,8 +277,10 @@ gst_gnome_vfs_sink_set_property (GObject * object, guint prop_id,
         g_free (sink->uri_name);
         sink->uri_name = NULL;
       }
-      if (g_value_get_string (value)) {
-        sink->uri_name = g_value_dup_string (value);
+
+      new_location = g_value_get_string (value);
+      if (new_location) {
+        sink->uri_name = gst_gnome_vfs_location_to_uri_string (new_location);
         sink->uri = gnome_vfs_uri_new (sink->uri_name);
       }
       break;
@@ -393,7 +398,7 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
     sink->own_handle = FALSE;
   }
 
-  sink->data_written = 0;
+  sink->current_pos = 0;
 
   return TRUE;
 }
@@ -469,6 +474,8 @@ gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
       if (res != GNOME_VFS_OK) {
         GST_ERROR_OBJECT (sink, "Failed to seek to offset %"
             G_GINT64_FORMAT ": %s", offset, gnome_vfs_result_to_string (res));
+      } else {
+        sink->current_pos = offset;
       }
 
       break;
@@ -500,7 +507,7 @@ gst_gnome_vfs_sink_query (GstPad * pad, GstQuery * query)
       switch (format) {
         case GST_FORMAT_DEFAULT:
         case GST_FORMAT_BYTES:
-          gst_query_set_position (query, GST_FORMAT_BYTES, sink->data_written);
+          gst_query_set_position (query, GST_FORMAT_BYTES, sink->current_pos);
           return TRUE;
         default:
           return FALSE;
@@ -522,13 +529,12 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
   GstGnomeVFSSink *sink;
   GnomeVFSResult result;
   GstFlowReturn ret;
-  guint64 back_pending = 0;
 
   sink = GST_GNOME_VFS_SINK (basesink);
 
   if (gnome_vfs_tell (sink->handle, &cur_pos) == GNOME_VFS_OK) {
-    if (cur_pos < sink->data_written)
-      back_pending = sink->data_written - cur_pos;
+    /* bring up to date with current position for proper reporting */
+    sink->current_pos = cur_pos;
   }
 
   result = gnome_vfs_write (sink->handle, GST_BUFFER_DATA (buf),
@@ -546,7 +552,7 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
             GST_BUFFER_SIZE (buf), written);
       }
 
-      sink->data_written += GST_BUFFER_SIZE (buf) - back_pending;
+      sink->current_pos += GST_BUFFER_SIZE (buf);
       ret = GST_FLOW_OK;
       break;
     }
@@ -578,7 +584,7 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_gnome_vfs_sink_uri_get_type (void)
 {
   return GST_URI_SINK;
