@@ -40,7 +40,7 @@ G_BEGIN_DECLS
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_GNOME_VFS_SRC,GstGnomeVFSSrcClass))
 #define GST_IS_GNOME_VFS_SRC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_GNOME_VFS_SRC))
-#define GST_IS_GNOME_VFS_SRC_CLASS(obj) \
+#define GST_IS_GNOME_VFS_SRC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_GNOME_VFS_SRC))
 
 typedef struct _GstGnomeVFSSrc      GstGnomeVFSSrc;
@@ -60,7 +60,6 @@ struct _GstGnomeVFSSrc
   gchar *uri_name;
   GnomeVFSHandle *handle;
   gboolean own_handle;
-  GnomeVFSFileSize size;        /* -1 = unknown */
   GnomeVFSFileOffset curoffset; /* current offset in file */
   gboolean seekable;
 
@@ -87,3 +86,4 @@ G_END_DECLS
 
 #endif /* __GST_GNOME_VFS_SRC_H__ */
 
+
