@@ -48,8 +48,8 @@
 #include "gsttheoradec.h"
 #include <gst/tag/tag.h>
 
-GST_DEBUG_CATEGORY (theoradec_debug);
 #define GST_CAT_DEFAULT theoradec_debug
+GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 #define THEORA_DEF_CROP         TRUE
 enum
@@ -58,7 +58,7 @@ enum
   ARG_CROP
 };
 
-static GstElementDetails theora_dec_details =
+static const GstElementDetails theora_dec_details =
 GST_ELEMENT_DETAILS ("Theora video decoder",
     "Codec/Decoder/Video",
     "decode raw theora streams to raw YUV video",
@@ -163,6 +163,8 @@ gst_theora_dec_init (GstTheoraDec * dec, GstTheoraDecClass * g_class)
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
   dec->crop = THEORA_DEF_CROP;
+  dec->gather = NULL;
+  dec->decode = NULL;
   dec->queued = NULL;
 }
 
@@ -170,17 +172,34 @@ static void
 gst_theora_dec_reset (GstTheoraDec * dec)
 {
   dec->need_keyframe = TRUE;
+  dec->sent_newsegment = FALSE;
   dec->last_timestamp = -1;
   dec->granulepos = -1;
+  dec->discont = TRUE;
+  dec->frame_nr = -1;
   gst_segment_init (&dec->segment, GST_FORMAT_TIME);
 
   GST_OBJECT_LOCK (dec);
   dec->proportion = 1.0;
   dec->earliest_time = -1;
   GST_OBJECT_UNLOCK (dec);
+
+  g_list_foreach (dec->queued, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (dec->queued);
+  dec->queued = NULL;
+  g_list_foreach (dec->gather, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (dec->gather);
+  dec->gather = NULL;
+  g_list_foreach (dec->decode, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (dec->decode);
+  dec->decode = NULL;
+
+  if (dec->tags) {
+    gst_tag_list_free (dec->tags);
+    dec->tags = NULL;
+  }
 }
 
-/* FIXME: copy from libtheora, theora should somehow make this available for seeking */
 static int
 _theora_ilog (unsigned int v)
 {
@@ -193,11 +212,13 @@ _theora_ilog (unsigned int v)
   return (ret);
 }
 
+/* Return the frame number (starting from zero) corresponding to this 
+ * granulepos */
 static gint64
 _theora_granule_frame (GstTheoraDec * dec, gint64 granulepos)
 {
   guint ilog;
-  gint framecount;
+  gint framenum;
 
   if (granulepos == -1)
     return -1;
@@ -206,18 +227,23 @@ _theora_granule_frame (GstTheoraDec * dec, gint64 granulepos)
 
   /* granulepos is last ilog bits for counting pframes since last iframe and 
    * bits in front of that for the framenumber of the last iframe. */
-  framecount = granulepos >> ilog;
-  framecount += granulepos - (framecount << ilog);
+  framenum = granulepos >> ilog;
+  framenum += granulepos - (framenum << ilog);
 
-  GST_DEBUG_OBJECT (dec, "framecount=%d, ilog=%u", framecount, ilog);
+  /* This is 1-based for current libtheora, 0 based for old. Fix up. */
+  if (!dec->is_old_bitstream)
+    framenum -= 1;
 
-  return framecount;
+  GST_DEBUG_OBJECT (dec, "framecount=%d, ilog=%u", framenum, ilog);
+
+  return framenum;
 }
 
+/* Return the frame start time corresponding to this granulepos */
 static GstClockTime
-_theora_granule_time (GstTheoraDec * dec, gint64 granulepos)
+_theora_granule_start_time (GstTheoraDec * dec, gint64 granulepos)
 {
-  gint framecount;
+  gint64 framecount;
 
   /* invalid granule results in invalid time */
   if (granulepos == -1)
@@ -281,6 +307,8 @@ theora_get_query_types (GstPad * pad)
 {
   static const GstQueryType theora_src_query_types[] = {
     GST_QUERY_POSITION,
+    GST_QUERY_DURATION,
+    GST_QUERY_CONVERT,
     0
   };
 
@@ -386,7 +414,7 @@ theora_dec_sink_convert (GstPad * pad,
     case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_TIME:
-          *dest_value = _theora_granule_time (dec, src_value);
+          *dest_value = _theora_granule_start_time (dec, src_value);
           break;
         default:
           res = FALSE;
@@ -463,7 +491,7 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
                   granulepos, &my_format, &time)))
         goto error;
 
-      time = (time - dec->segment.start) + dec->segment.time;
+      time = gst_segment_to_stream_time (&dec->segment, GST_FORMAT_TIME, time);
 
       GST_LOG_OBJECT (dec,
           "query %p: our time: %" GST_TIME_FORMAT, query, GST_TIME_ARGS (time));
@@ -480,10 +508,20 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
       break;
     }
     case GST_QUERY_DURATION:
-      /* forward to peer for total */
-      if (!(res = gst_pad_query (GST_PAD_PEER (dec->sinkpad), query)))
+    {
+      GstPad *peer;
+
+      if (!(peer = gst_pad_get_peer (dec->sinkpad)))
         goto error;
+
+      /* forward to peer for total */
+      res = gst_pad_query (peer, query);
+      gst_object_unref (peer);
+      if (!res)
+        goto error;
+
       break;
+    }
     case GST_QUERY_CONVERT:
     {
       GstFormat src_fmt, dest_fmt;
@@ -584,7 +622,6 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
           flags, cur_type, tcur, stop_type, tstop);
 
       res = gst_pad_push_event (dec->sinkpad, real_seek);
-
       break;
     }
     case GST_EVENT_QOS:
@@ -641,7 +678,6 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
     case GST_EVENT_FLUSH_STOP:
-      /* FIXME, makes us waiting for keyframes */
       gst_theora_dec_reset (dec);
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
@@ -652,25 +688,28 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
     {
       gboolean update;
       GstFormat format;
-      gdouble rate;
+      gdouble rate, arate;
       gint64 start, stop, time;
 
-      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
-          &stop, &time);
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &format,
+          &start, &stop, &time);
 
-      /* we need TIME and a positive rate */
+      /* we need TIME format */
       if (format != GST_FORMAT_TIME)
         goto newseg_wrong_format;
 
-      if (rate <= 0.0)
-        goto newseg_wrong_rate;
-
       /* now configure the values */
-      gst_segment_set_newsegment (&dec->segment, update,
-          rate, format, start, stop, time);
+      gst_segment_set_newsegment_full (&dec->segment, update,
+          rate, arate, format, start, stop, time);
 
-      /* and forward */
-      ret = gst_pad_push_event (dec->srcpad, event);
+      /* We don't forward this unless/until the decoder is initialised */
+      if (dec->have_header) {
+        ret = gst_pad_push_event (dec->srcpad, event);
+        dec->sent_newsegment = TRUE;
+      } else {
+        gst_event_unref (event);
+        ret = TRUE;
+      }
       break;
     }
     default:
@@ -689,18 +728,8 @@ newseg_wrong_format:
     gst_event_unref (event);
     goto done;
   }
-newseg_wrong_rate:
-  {
-    GST_DEBUG_OBJECT (dec, "negative rates not supported yet");
-    gst_event_unref (event);
-    goto done;
-  }
 }
 
-#define ROUND_UP_2(x) (((x) + 1) & ~1)
-#define ROUND_UP_4(x) (((x) + 3) & ~3)
-#define ROUND_UP_8(x) (((x) + 7) & ~7)
-
 static GstFlowReturn
 theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
@@ -738,7 +767,7 @@ theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
         GST_TAG_NOMINAL_BITRATE, dec->info.target_bitrate, NULL);
   }
 
-  gst_element_found_tags_for_pad (GST_ELEMENT (dec), dec->srcpad, list);
+  dec->tags = list;
 
   return GST_FLOW_OK;
 }
@@ -748,6 +777,10 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
   GstCaps *caps;
   gint par_num, par_den;
+  GstFlowReturn ret = GST_FLOW_OK;
+  gboolean eret;
+  GstEvent *event;
+  guint32 bitstream_version;
 
   GST_DEBUG_OBJECT (dec, "fps %d/%d, PAR %d/%d",
       dec->info.fps_numerator, dec->info.fps_denominator,
@@ -772,13 +805,20 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   GST_DEBUG_OBJECT (dec, "frame dimension %dx%d, offset %d:%d",
       dec->info.frame_width, dec->info.frame_height,
       dec->info.offset_x, dec->info.offset_y);
+  if (dec->info.pixelformat != OC_PF_420) {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("pixel formats other than 4:2:0 not yet supported"));
+
+    return GST_FLOW_ERROR;
+  }
 
   if (dec->crop) {
     /* add black borders to make width/height/offsets even. we need this because
      * we cannot express an offset to the peer plugin. */
-    dec->width = ROUND_UP_2 (dec->info.frame_width + (dec->info.offset_x & 1));
+    dec->width =
+        GST_ROUND_UP_2 (dec->info.frame_width + (dec->info.offset_x & 1));
     dec->height =
-        ROUND_UP_2 (dec->info.frame_height + (dec->info.offset_y & 1));
+        GST_ROUND_UP_2 (dec->info.frame_height + (dec->info.offset_y & 1));
     dec->offset_x = dec->info.offset_x & ~1;
     dec->offset_y = dec->info.offset_y & ~1;
   } else {
@@ -791,6 +831,16 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
 
   dec->granule_shift = _theora_ilog (dec->info.keyframe_frequency_force - 1);
 
+  /* With libtheora-1.0beta1 the granulepos scheme was changed:
+   * where earlier the granulepos refered to the index/beginning
+   * of a frame, it now refers to the end, which matches the use
+   * in vorbis/speex. We check the bitstream version from the header so
+   * we know which way to interpret the incoming granuepos
+   */
+  bitstream_version = (dec->info.version_major << 16) |
+      (dec->info.version_minor << 8) | dec->info.version_subminor;
+  dec->is_old_bitstream = (bitstream_version <= 0x00030200);
+
   GST_DEBUG_OBJECT (dec, "after fixup frame dimension %dx%d, offset %d:%d",
       dec->width, dec->height, dec->offset_x, dec->offset_y);
 
@@ -807,8 +857,26 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   gst_caps_unref (caps);
 
   dec->have_header = TRUE;
+  if (!dec->sent_newsegment) {
+    GST_DEBUG_OBJECT (dec, "Sending newsegment event");
 
-  return GST_FLOW_OK;
+    event = gst_event_new_new_segment_full (FALSE,
+        dec->segment.rate, dec->segment.applied_rate,
+        dec->segment.format, dec->segment.start, dec->segment.stop,
+        dec->segment.time);
+    eret = gst_pad_push_event (dec->srcpad, event);
+    if (!eret)
+      ret = GST_FLOW_ERROR;
+    dec->sent_newsegment = TRUE;
+  }
+
+  if (dec->tags) {
+    gst_element_found_tags_for_pad (GST_ELEMENT_CAST (dec), dec->srcpad,
+        dec->tags);
+    dec->tags = NULL;
+  }
+
+  return ret;
 }
 
 static GstFlowReturn
@@ -847,17 +915,60 @@ header_read_error:
   }
 }
 
+/* returns TRUE if buffer is within segment, else FALSE.
+ * if Buffer is on segment border, it's timestamp and duration will be clipped */
+static gboolean
+clip_buffer (GstTheoraDec * dec, GstBuffer * buf)
+{
+  gboolean res = TRUE;
+  GstClockTime in_ts, in_dur, stop;
+  gint64 cstart, cstop;
+
+  in_ts = GST_BUFFER_TIMESTAMP (buf);
+  in_dur = GST_BUFFER_DURATION (buf);
+
+  GST_LOG_OBJECT (dec,
+      "timestamp:%" GST_TIME_FORMAT " , duration:%" GST_TIME_FORMAT,
+      GST_TIME_ARGS (in_ts), GST_TIME_ARGS (in_dur));
+
+  /* can't clip without TIME segment */
+  if (dec->segment.format != GST_FORMAT_TIME)
+    goto beach;
+
+  /* we need a start time */
+  if (!GST_CLOCK_TIME_IS_VALID (in_ts))
+    goto beach;
+
+  /* generate valid stop, if duration unknown, we have unknown stop */
+  stop =
+      GST_CLOCK_TIME_IS_VALID (in_dur) ? (in_ts + in_dur) : GST_CLOCK_TIME_NONE;
+
+  /* now clip */
+  if (!(res = gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
+              in_ts, stop, &cstart, &cstop)))
+    goto beach;
+
+  /* update timestamp and possibly duration if the clipped stop time is
+   * valid */
+  GST_BUFFER_TIMESTAMP (buf) = cstart;
+  if (GST_CLOCK_TIME_IS_VALID (cstop))
+    GST_BUFFER_DURATION (buf) = cstop - cstart;
+
+beach:
+  GST_LOG_OBJECT (dec, "%sdropping", (res ? "not " : ""));
+  return res;
+}
+
 /* FIXME, this needs to be moved to the demuxer */
 static GstFlowReturn
-theora_dec_push (GstTheoraDec * dec, GstBuffer * buf)
+theora_dec_push_forward (GstTheoraDec * dec, GstBuffer * buf)
 {
-  GstFlowReturn result;
+  GstFlowReturn result = GST_FLOW_OK;
   GstClockTime outtime = GST_BUFFER_TIMESTAMP (buf);
 
   if (outtime == GST_CLOCK_TIME_NONE) {
     dec->queued = g_list_append (dec->queued, buf);
     GST_DEBUG_OBJECT (dec, "queued buffer");
-    result = GST_FLOW_OK;
   } else {
     if (dec->queued) {
       gint64 size;
@@ -876,15 +987,43 @@ theora_dec_push (GstTheoraDec * dec, GstBuffer * buf)
 
         GST_DEBUG_OBJECT (dec, "patch buffer %lld %lld", size, time);
         GST_BUFFER_TIMESTAMP (buffer) = time;
+        /* Next timestamp - this one is duration */
+        GST_BUFFER_DURATION (buffer) =
+            (outtime - gst_util_uint64_scale_int ((size - 1) * GST_SECOND,
+                dec->info.fps_denominator, dec->info.fps_numerator)) - time;
+
+        if (dec->discont) {
+          GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
+          dec->discont = FALSE;
+        }
         /* ignore the result.. */
-        gst_pad_push (dec->srcpad, buffer);
+        if (clip_buffer (dec, buffer))
+          gst_pad_push (dec->srcpad, buffer);
+        else
+          gst_buffer_unref (buffer);
         size--;
       }
       g_list_free (dec->queued);
       dec->queued = NULL;
     }
-    result = gst_pad_push (dec->srcpad, buf);
+    if (dec->discont) {
+      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+      dec->discont = FALSE;
+    }
+    if (clip_buffer (dec, buf))
+      result = gst_pad_push (dec->srcpad, buf);
+    else
+      gst_buffer_unref (buf);
   }
+  return result;
+}
+
+static GstFlowReturn
+theora_dec_push_reverse (GstTheoraDec * dec, GstBuffer * buf)
+{
+  GstFlowReturn result = GST_FLOW_OK;
+
+  dec->queued = g_list_prepend (dec->queued, buf);
 
   return result;
 }
@@ -904,20 +1043,23 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   gint cwidth, cheight;
   GstFlowReturn result;
 
-  if (!dec->have_header)
+  if (G_UNLIKELY (!dec->have_header))
     goto not_initialized;
 
   /* the second most significant bit of the first data byte is cleared 
-   * for keyframes */
-  keyframe = (packet->packet[0] & 0x40) == 0;
-  if (keyframe) {
+   * for keyframes. We can only check it if it's not a zero-length packet. */
+  keyframe = packet->bytes && ((packet->packet[0] & 0x40) == 0);
+  if (G_UNLIKELY (keyframe)) {
+    GST_DEBUG_OBJECT (dec, "we have a keyframe");
     dec->need_keyframe = FALSE;
-  } else if (dec->need_keyframe) {
+  } else if (G_UNLIKELY (dec->need_keyframe)) {
     goto dropping;
   }
 
+  GST_DEBUG_OBJECT (dec, "parsing data packet");
+
   /* this does the decoding */
-  if (theora_decode_packetin (&dec->state, packet))
+  if (G_UNLIKELY (theora_decode_packetin (&dec->state, packet)))
     goto decode_error;
 
   if (outtime != -1) {
@@ -942,10 +1084,11 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
 
   /* this does postprocessing and set up the decoded frame
    * pointers in our yuv variable */
-  if (theora_decode_YUVout (&dec->state, &yuv) < 0)
+  if (G_UNLIKELY (theora_decode_YUVout (&dec->state, &yuv) < 0))
     goto no_yuv;
 
-  if ((yuv.y_width != dec->info.width) || (yuv.y_height != dec->info.height))
+  if (G_UNLIKELY ((yuv.y_width != dec->info.width)
+          || (yuv.y_height != dec->info.height)))
     goto wrong_dimensions;
 
   width = dec->width;
@@ -956,8 +1099,8 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   /* should get the stride from the caps, for now we round up to the nearest
    * multiple of 4 because some element needs it. chroma needs special 
    * treatment, see videotestsrc. */
-  stride_y = ROUND_UP_4 (width);
-  stride_uv = ROUND_UP_8 (width) / 2;
+  stride_y = GST_ROUND_UP_4 (width);
+  stride_uv = GST_ROUND_UP_8 (width) / 2;
 
   out_size = stride_y * height + stride_uv * cheight * 2;
 
@@ -966,7 +1109,7 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   result =
       gst_pad_alloc_buffer_and_set_caps (dec->srcpad, GST_BUFFER_OFFSET_NONE,
       out_size, GST_PAD_CAPS (dec->srcpad), &out);
-  if (result != GST_FLOW_OK)
+  if (G_UNLIKELY (result != GST_FLOW_OK))
     goto no_buffer;
 
   /* copy the visible region to the destination. This is actually pretty
@@ -1011,16 +1154,26 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     }
   }
 
-  /* FIXME, frame_nr not correct */
   GST_BUFFER_OFFSET (out) = dec->frame_nr;
-  dec->frame_nr++;
+  if (dec->frame_nr != -1)
+    dec->frame_nr++;
   GST_BUFFER_OFFSET_END (out) = dec->frame_nr;
-  GST_BUFFER_DURATION (out) =
-      gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
-      dec->info.fps_numerator);
+  if (dec->granulepos != -1) {
+    gint64 cf = _theora_granule_frame (dec, dec->granulepos) + 1;
+
+    GST_BUFFER_DURATION (out) = gst_util_uint64_scale_int (cf * GST_SECOND,
+        dec->info.fps_denominator, dec->info.fps_numerator) - outtime;
+  } else {
+    GST_BUFFER_DURATION (out) =
+        gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
+        dec->info.fps_numerator);
+  }
   GST_BUFFER_TIMESTAMP (out) = outtime;
 
-  result = theora_dec_push (dec, out);
+  if (dec->segment.rate >= 0.0)
+    result = theora_dec_push_forward (dec, out);
+  else
+    result = theora_dec_push_reverse (dec, out);
 
   return result;
 
@@ -1034,11 +1187,14 @@ not_initialized:
 dropping:
   {
     GST_WARNING_OBJECT (dec, "dropping frame because we need a keyframe");
+    dec->discont = TRUE;
     return GST_FLOW_OK;
   }
 dropping_qos:
   {
-    dec->frame_nr++;
+    if (dec->frame_nr != -1)
+      dec->frame_nr++;
+    dec->discont = TRUE;
     GST_WARNING_OBJECT (dec, "dropping frame because of QoS");
     return GST_FLOW_OK;
   }
@@ -1069,21 +1225,11 @@ no_buffer:
 }
 
 static GstFlowReturn
-theora_dec_chain (GstPad * pad, GstBuffer * buf)
+theora_dec_decode_buffer (GstTheoraDec * dec, GstBuffer * buf)
 {
-  GstTheoraDec *dec;
   ogg_packet packet;
   GstFlowReturn result = GST_FLOW_OK;
 
-  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
-
-  /* resync on DISCONT */
-  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
-    dec->need_keyframe = TRUE;
-    dec->last_timestamp = -1;
-    dec->granulepos = -1;
-  }
-
   /* make ogg_packet out of the buffer */
   packet.packet = GST_BUFFER_DATA (buf);
   packet.bytes = GST_BUFFER_SIZE (buf);
@@ -1096,22 +1242,24 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
   if (dec->have_header) {
     if (packet.granulepos != -1) {
       dec->granulepos = packet.granulepos;
-      dec->last_timestamp = _theora_granule_time (dec, packet.granulepos);
+      dec->last_timestamp = _theora_granule_start_time (dec, packet.granulepos);
     } else if (dec->last_timestamp != -1) {
-      dec->last_timestamp = _theora_granule_time (dec, dec->granulepos);
+      dec->last_timestamp = _theora_granule_start_time (dec, dec->granulepos);
     }
+    if (dec->last_timestamp == -1 && GST_BUFFER_TIMESTAMP_IS_VALID (buf))
+      dec->last_timestamp = GST_BUFFER_TIMESTAMP (buf);
   } else {
     dec->last_timestamp = -1;
   }
 
-  if (packet.bytes < 1)
-    goto wrong_size;
+  GST_DEBUG_OBJECT (dec, "header=%02x packetno=%lld, granule pos=%"
+      G_GINT64_FORMAT ", outtime=%" GST_TIME_FORMAT,
+      packet.bytes ? packet.packet[0] : -1, packet.packetno, packet.granulepos,
+      GST_TIME_ARGS (dec->last_timestamp));
 
-  GST_DEBUG_OBJECT (dec, "header=%d packetno=%lld, outtime=%" GST_TIME_FORMAT,
-      packet.packet[0], packet.packetno, GST_TIME_ARGS (dec->last_timestamp));
-
-  /* switch depending on packet type */
-  if (packet.packet[0] & 0x80) {
+  /* switch depending on packet type. A zero byte packet is always a data
+   * packet; we don't dereference it in that case. */
+  if (packet.bytes && packet.packet[0] & 0x80) {
     if (dec->have_header) {
       GST_WARNING_OBJECT (GST_OBJECT (dec), "Ignoring header");
       goto done;
@@ -1122,22 +1270,205 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
   }
 
 done:
-  /* interpollate granule pos */
+  /* interpolate granule pos */
   dec->granulepos = _inc_granulepos (dec, dec->granulepos);
 
+  return result;
+}
+
+/* For reverse playback we use a technique that can be used for
+ * any keyframe based video codec.
+ *
+ * Input:
+ *  Buffer decoding order:  7  8  9  4  5  6  1  2  3  EOS
+ *  Keyframe flag:                      K        K
+ *  Discont flag:           D        D        D
+ *
+ * - Each Discont marks a discont in the decoding order.
+ * - The keyframes mark where we can start decoding.
+ *
+ * First we prepend incomming buffers to the gather queue, whenever we receive
+ * a discont, we flush out the gather queue.
+ *
+ * The above data will be accumulated in the gather queue like this:
+ *
+ *   gather queue:  9  8  7
+ *                        D
+ *
+ * Whe buffer 4 is received (with a DISCONT), we flush the gather queue like
+ * this:
+ *
+ *   while (gather)
+ *     take head of queue and prepend to decode queue.
+ *     if we copied a keyframe, decode the decode queue.
+ *
+ * After we flushed the gather queue, we add 4 to the (now empty) gather queue.
+ * We get the following situation:
+ *
+ *  gather queue:    4
+ *  decode queue:    7  8  9
+ *
+ * After we received 5 (Keyframe) and 6:
+ *
+ *  gather queue:    6  5  4
+ *  decode queue:    7  8  9
+ *
+ * When we receive 1 (DISCONT) which triggers a flush of the gather queue:
+ *
+ *   Copy head of the gather queue (6) to decode queue:
+ *
+ *    gather queue:    5  4
+ *    decode queue:    6  7  8  9
+ *
+ *   Copy head of the gather queue (5) to decode queue. This is a keyframe so we
+ *   can start decoding.
+ *
+ *    gather queue:    4
+ *    decode queue:    5  6  7  8  9
+ *
+ *   Decode frames in decode queue, store raw decoded data in output queue, we
+ *   can take the head of the decode queue and prepend the decoded result in the
+ *   output queue:
+ *
+ *    gather queue:    4
+ *    decode queue:    
+ *    output queue:    9  8  7  6  5
+ *
+ *   Now output all the frames in the output queue, picking a frame from the
+ *   head of the queue.
+ *
+ *   Copy head of the gather queue (4) to decode queue, we flushed the gather
+ *   queue and can now store input buffer in the gather queue:
+ *
+ *    gather queue:    1
+ *    decode queue:    4
+ *
+ *  When we receive EOS, the queue looks like:
+ *
+ *    gather queue:    3  2  1
+ *    decode queue:    4
+ *
+ *  Fill decode queue, first keyframe we copy is 2:
+ *
+ *    gather queue:    1
+ *    decode queue:    2  3  4
+ *
+ *  Decoded output:
+ *
+ *    gather queue:    1
+ *    decode queue:    
+ *    output queue:    4  3  2
+ *
+ *  Leftover buffer 1 cannot be decoded and must be discarded.
+ */
+static GstFlowReturn
+theora_dec_flush_decode (GstTheoraDec * dec)
+{
+  GstFlowReturn res = GST_FLOW_OK;
+
+  while (dec->decode) {
+    GstBuffer *buf = GST_BUFFER_CAST (dec->decode->data);
+
+    GST_DEBUG_OBJECT (dec, "decoding buffer %p, ts %" GST_TIME_FORMAT,
+        buf, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+
+    /* decode buffer, prepend to output queue */
+    res = theora_dec_decode_buffer (dec, buf);
+
+    /* don't need it anymore now */
+    gst_buffer_unref (buf);
+
+    dec->decode = g_list_delete_link (dec->decode, dec->decode);
+  }
+  while (dec->queued) {
+    GstBuffer *buf = GST_BUFFER_CAST (dec->queued->data);
+
+    /* iterate ouput queue an push downstream */
+    res = gst_pad_push (dec->srcpad, buf);
+
+    dec->queued = g_list_delete_link (dec->queued, dec->queued);
+  }
+
+  return res;
+}
+
+static GstFlowReturn
+theora_dec_chain_reverse (GstTheoraDec * dec, gboolean discont, GstBuffer * buf)
+{
+  GstFlowReturn res = GST_FLOW_OK;
+
+  /* if we have a discont, move buffers to the decode list */
+  if (G_UNLIKELY (discont)) {
+    GST_DEBUG_OBJECT (dec, "received discont,gathering buffers");
+    while (dec->gather) {
+      GstBuffer *gbuf;
+      guint8 *data;
+
+      gbuf = GST_BUFFER_CAST (dec->gather->data);
+      /* remove from the gather list */
+      dec->gather = g_list_delete_link (dec->gather, dec->gather);
+      /* copy to decode queue */
+      dec->decode = g_list_prepend (dec->decode, gbuf);
+
+      /* if we copied a keyframe, flush and decode the decode queue */
+      data = GST_BUFFER_DATA (gbuf);
+      if ((data[0] & 0x40) == 0) {
+        GST_DEBUG_OBJECT (dec, "copied keyframe");
+        res = theora_dec_flush_decode (dec);
+      }
+    }
+  }
+
+  /* add buffer to gather queue */
+  GST_DEBUG_OBJECT (dec, "gathering buffer %p, size %u", buf,
+      GST_BUFFER_SIZE (buf));
+  dec->gather = g_list_prepend (dec->gather, buf);
+
+  return res;
+}
+
+static GstFlowReturn
+theora_dec_chain_forward (GstTheoraDec * dec, gboolean discont,
+    GstBuffer * buffer)
+{
+  GstFlowReturn result;
+
+  result = theora_dec_decode_buffer (dec, buffer);
+
+  gst_buffer_unref (buffer);
+
+  return result;
+}
+
+static GstFlowReturn
+theora_dec_chain (GstPad * pad, GstBuffer * buf)
+{
+  GstTheoraDec *dec;
+  GstFlowReturn res;
+  gboolean discont;
+
+  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+
+  /* peel of DISCONT flag */
+  discont = GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT);
+
+  /* resync on DISCONT */
+  if (G_UNLIKELY (discont)) {
+    GST_DEBUG_OBJECT (dec, "received DISCONT buffer");
+    dec->need_keyframe = TRUE;
+    dec->last_timestamp = -1;
+    dec->granulepos = -1;
+    dec->discont = TRUE;
+  }
+
+  if (dec->segment.rate > 0.0)
+    res = theora_dec_chain_forward (dec, discont, buf);
+  else
+    res = theora_dec_chain_reverse (dec, discont, buf);
+
   gst_object_unref (dec);
 
-  gst_buffer_unref (buf);
-
-  return result;
-
-  /* ERRORS */
-wrong_size:
-  {
-    GST_WARNING_OBJECT (dec, "received empty packet");
-    result = GST_FLOW_OK;
-    goto done;
-  }
+  return res;
 }
 
 static GstStateChangeReturn
@@ -1146,13 +1477,13 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
   GstTheoraDec *dec = GST_THEORA_DEC (element);
   GstStateChangeReturn ret;
 
-
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       theora_info_init (&dec->info);
       theora_comment_init (&dec->comment);
+      GST_DEBUG_OBJECT (dec, "Setting have_header to FALSE in READY->PAUSED");
       dec->have_header = FALSE;
       gst_theora_dec_reset (dec);
       break;
@@ -1171,6 +1502,7 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
       theora_clear (&dec->state);
       theora_comment_clear (&dec->comment);
       theora_info_clear (&dec->info);
+      gst_theora_dec_reset (dec);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
