@@ -19,6 +19,25 @@
  */
 /* Element-Checklist-Version: 5 */
 
+/**
+ * SECTION:element-audioresample
+ *
+ * <refsect2>
+ * Audioresample resamples raw audio buffers to different sample rates using
+ * a configurable windowing function to enhance quality.
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! audio/x-raw-int, rate=8000 ! alsasink
+ * </programlisting>
+ * Decode an Ogg/Vorbis downsample to 8Khz and play sound through alsa. 
+ * To create the Ogg/Vorbis file refer to the documentation of vorbisenc.
+ * </para>
+ * </refsect2>
+ *
+ * Last reviewed on 2006-03-02 (0.10.4)
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -35,25 +54,18 @@ GST_DEBUG_CATEGORY (audioresample_debug);
 #define GST_CAT_DEFAULT audioresample_debug
 
 /* elementfactory information */
-static GstElementDetails gst_audioresample_details =
+static const GstElementDetails gst_audioresample_details =
 GST_ELEMENT_DETAILS ("Audio scaler",
     "Filter/Converter/Audio",
     "Resample audio",
     "David Schleef <ds@schleef.org>");
 
-/* GstAudioresample signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
 #define DEFAULT_FILTERLEN       16
 
 enum
 {
-  ARG_0,
-  ARG_FILTERLEN
+  PROP_0,
+  PROP_FILTERLEN
 };
 
 #define SUPPORTED_CAPS \
@@ -64,49 +76,59 @@ GST_STATIC_CAPS ( \
       "endianness = (int) BYTE_ORDER, " \
       "width = (int) 16, " \
       "depth = (int) 16, " \
-      "signed = (boolean) true " \
+      "signed = (boolean) true;" \
+    "audio/x-raw-int, " \
+      "rate = (int) [ 1, MAX ], " \
+      "channels = (int) [ 1, MAX ], " \
+      "endianness = (int) BYTE_ORDER, " \
+      "width = (int) 32, " \
+      "depth = (int) 32, " \
+      "signed = (boolean) true;" \
+    "audio/x-raw-float, " \
+      "rate = (int) [ 1, MAX ], "	\
+      "channels = (int) [ 1, MAX ], " \
+      "endianness = (int) BYTE_ORDER, " \
+      "width = (int) 32; " \
+    "audio/x-raw-float, " \
+      "rate = (int) [ 1, MAX ], "	\
+      "channels = (int) [ 1, MAX ], " \
+      "endianness = (int) BYTE_ORDER, " \
+      "width = (int) 64" \
 )
 
-#if 0
-  /* disabled because it segfaults */
-"audio/x-raw-float, "
-    "rate = (int) [ 1, MAX ], "
-    "channels = (int) [ 1, MAX ], "
-    "endianness = (int) BYTE_ORDER, " "width = (int) 32")
-#endif
-     static GstStaticPadTemplate gst_audioresample_sink_template =
-         GST_STATIC_PAD_TEMPLATE ("sink",
-         GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+static GstStaticPadTemplate gst_audioresample_sink_template =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
-     static GstStaticPadTemplate gst_audioresample_src_template =
-         GST_STATIC_PAD_TEMPLATE ("src",
-         GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
+static GstStaticPadTemplate gst_audioresample_src_template =
+GST_STATIC_PAD_TEMPLATE ("src",
+    GST_PAD_SRC, GST_PAD_ALWAYS, SUPPORTED_CAPS);
 
-     static void gst_audioresample_dispose (GObject * object);
-
-     static void gst_audioresample_set_property (GObject * object,
-         guint prop_id, const GValue * value, GParamSpec * pspec);
-     static void gst_audioresample_get_property (GObject * object,
-         guint prop_id, GValue * value, GParamSpec * pspec);
+static void gst_audioresample_set_property (GObject * object,
+    guint prop_id, const GValue * value, GParamSpec * pspec);
+static void gst_audioresample_get_property (GObject * object,
+    guint prop_id, GValue * value, GParamSpec * pspec);
 
 /* vmethods */
-     gboolean audioresample_get_unit_size (GstBaseTransform * base,
-         GstCaps * caps, guint * size);
-     GstCaps *audioresample_transform_caps (GstBaseTransform * base,
-         GstPadDirection direction, GstCaps * caps);
-     gboolean audioresample_transform_size (GstBaseTransform * trans,
-         GstPadDirection direction, GstCaps * incaps, guint insize,
-         GstCaps * outcaps, guint * outsize);
-     gboolean audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
-         GstCaps * outcaps);
-     static GstFlowReturn audioresample_pushthrough (GstAudioresample *
-         audioresample);
-     static GstFlowReturn audioresample_transform (GstBaseTransform * base,
-         GstBuffer * inbuf, GstBuffer * outbuf);
-     static gboolean audioresample_event (GstBaseTransform * base,
-         GstEvent * event);
+static gboolean audioresample_get_unit_size (GstBaseTransform * base,
+    GstCaps * caps, guint * size);
+static GstCaps *audioresample_transform_caps (GstBaseTransform * base,
+    GstPadDirection direction, GstCaps * caps);
+static gboolean audioresample_transform_size (GstBaseTransform * trans,
+    GstPadDirection direction, GstCaps * incaps, guint insize,
+    GstCaps * outcaps, guint * outsize);
+static gboolean audioresample_set_caps (GstBaseTransform * base,
+    GstCaps * incaps, GstCaps * outcaps);
+static GstFlowReturn audioresample_pushthrough (GstAudioresample *
+    audioresample);
+static GstFlowReturn audioresample_transform (GstBaseTransform * base,
+    GstBuffer * inbuf, GstBuffer * outbuf);
+static gboolean audioresample_event (GstBaseTransform * base, GstEvent * event);
+static gboolean audioresample_start (GstBaseTransform * base);
+static gboolean audioresample_stop (GstBaseTransform * base);
 
-/*static guint gst_audioresample_signals[LAST_SIGNAL] = { 0 }; */
+static gboolean audioresample_query (GstPad * pad, GstQuery * query);
+static const GstQueryType *audioresample_query_type (GstPad * pad);
 
 #define DEBUG_INIT(bla) \
   GST_DEBUG_CATEGORY_INIT (audioresample_debug, "audioresample", 0, "audio resampling element");
@@ -114,20 +136,21 @@ GST_STATIC_CAPS ( \
 GST_BOILERPLATE_FULL (GstAudioresample, gst_audioresample, GstBaseTransform,
     GST_TYPE_BASE_TRANSFORM, DEBUG_INIT);
 
-     static void gst_audioresample_base_init (gpointer g_class)
-     {
-       GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+static void
+gst_audioresample_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
 
-       gst_element_class_add_pad_template (gstelement_class,
-           gst_static_pad_template_get (&gst_audioresample_src_template));
-       gst_element_class_add_pad_template (gstelement_class,
-           gst_static_pad_template_get (&gst_audioresample_sink_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_audioresample_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_audioresample_sink_template));
 
-       gst_element_class_set_details (gstelement_class,
-           &gst_audioresample_details);
-     }
+  gst_element_class_set_details (gstelement_class, &gst_audioresample_details);
+}
 
-static void gst_audioresample_class_init (GstAudioresampleClass * klass)
+static void
+gst_audioresample_class_init (GstAudioresampleClass * klass)
 {
   GObjectClass *gobject_class;
 
@@ -135,13 +158,16 @@ static void gst_audioresample_class_init (GstAudioresampleClass * klass)
 
   gobject_class->set_property = gst_audioresample_set_property;
   gobject_class->get_property = gst_audioresample_get_property;
-  gobject_class->dispose = gst_audioresample_dispose;
 
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FILTERLEN,
+  g_object_class_install_property (gobject_class, PROP_FILTERLEN,
       g_param_spec_int ("filter_length", "filter_length", "filter_length",
           0, G_MAXINT, DEFAULT_FILTERLEN,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+  GST_BASE_TRANSFORM_CLASS (klass)->start =
+      GST_DEBUG_FUNCPTR (audioresample_start);
+  GST_BASE_TRANSFORM_CLASS (klass)->stop =
+      GST_DEBUG_FUNCPTR (audioresample_stop);
   GST_BASE_TRANSFORM_CLASS (klass)->transform_size =
       GST_DEBUG_FUNCPTR (audioresample_transform_size);
   GST_BASE_TRANSFORM_CLASS (klass)->get_unit_size =
@@ -159,49 +185,67 @@ static void gst_audioresample_class_init (GstAudioresampleClass * klass)
 }
 
 static void
-    gst_audioresample_init (GstAudioresample * audioresample,
+gst_audioresample_init (GstAudioresample * audioresample,
     GstAudioresampleClass * klass)
 {
-  ResampleState *r;
   GstBaseTransform *trans;
 
   trans = GST_BASE_TRANSFORM (audioresample);
 
   /* buffer alloc passthrough is too impossible. FIXME, it
-   * is trivial in the passtrough case. */
+   * is trivial in the passthrough case. */
   gst_pad_set_bufferalloc_function (trans->sinkpad, NULL);
 
-  r = resample_new ();
-  audioresample->resample = r;
+  audioresample->filter_length = DEFAULT_FILTERLEN;
+
+  audioresample->need_discont = FALSE;
+
+  gst_pad_set_query_function (trans->srcpad, audioresample_query);
+  gst_pad_set_query_type_function (trans->srcpad, audioresample_query_type);
+}
+
+/* vmethods */
+static gboolean
+audioresample_start (GstBaseTransform * base)
+{
+  GstAudioresample *audioresample = GST_AUDIORESAMPLE (base);
+
+  audioresample->resample = resample_new ();
   audioresample->ts_offset = -1;
   audioresample->offset = -1;
   audioresample->next_ts = -1;
 
-  resample_set_filter_length (r, DEFAULT_FILTERLEN);
-  resample_set_format (r, RESAMPLE_FORMAT_S16);
+  resample_set_filter_length (audioresample->resample,
+      audioresample->filter_length);
+
+  return TRUE;
 }
 
-static void gst_audioresample_dispose (GObject * object)
+static gboolean
+audioresample_stop (GstBaseTransform * base)
 {
-  GstAudioresample *audioresample = GST_AUDIORESAMPLE (object);
+  GstAudioresample *audioresample = GST_AUDIORESAMPLE (base);
 
   if (audioresample->resample) {
     resample_free (audioresample->resample);
     audioresample->resample = NULL;
   }
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  gst_caps_replace (&audioresample->sinkcaps, NULL);
+  gst_caps_replace (&audioresample->srccaps, NULL);
+
+  return TRUE;
 }
 
-/* vmethods */
-gboolean
-    audioresample_get_unit_size (GstBaseTransform * base, GstCaps * caps,
-    guint * size) {
+static gboolean
+audioresample_get_unit_size (GstBaseTransform * base, GstCaps * caps,
+    guint * size)
+{
   gint width, channels;
   GstStructure *structure;
   gboolean ret;
 
-  g_return_val_if_fail (size, FALSE);
+  g_assert (size);
 
   /* this works for both float and int */
   structure = gst_caps_get_structure (caps, 0);
@@ -214,7 +258,8 @@ gboolean
   return TRUE;
 }
 
-GstCaps *audioresample_transform_caps (GstBaseTransform * base,
+static GstCaps *
+audioresample_transform_caps (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps)
 {
   GstCaps *res;
@@ -230,35 +275,56 @@ GstCaps *audioresample_transform_caps (GstBaseTransform * base,
 }
 
 static gboolean
-    resample_set_state_from_caps (ResampleState * state, GstCaps * incaps,
+resample_set_state_from_caps (ResampleState * state, GstCaps * incaps,
     GstCaps * outcaps, gint * channels, gint * inrate, gint * outrate)
 {
   GstStructure *structure;
   gboolean ret;
   gint myinrate, myoutrate;
   int mychannels;
+  gint width, depth;
+  ResampleFormat format;
 
   GST_DEBUG ("incaps %" GST_PTR_FORMAT ", outcaps %"
       GST_PTR_FORMAT, incaps, outcaps);
 
   structure = gst_caps_get_structure (incaps, 0);
 
-  /* FIXME: once it does float, set the correct format */
-#if 0
-  if (g_str_equal (gst_structure_get_name (structure), "audio/x-raw-float")) {
-    r->format = GST_RESAMPLE_FLOAT;
-  } else {
-    r->format = GST_RESAMPLE_S16;
-  }
-#endif
+  /* get width */
+  ret = gst_structure_get_int (structure, "width", &width);
+  if (!ret)
+    goto no_width;
 
+  /* figure out the format */
+  if (g_str_equal (gst_structure_get_name (structure), "audio/x-raw-float")) {
+    if (width == 32)
+      format = RESAMPLE_FORMAT_F32;
+    else if (width == 64)
+      format = RESAMPLE_FORMAT_F64;
+    else
+      goto wrong_depth;
+  } else {
+    /* for int, depth and width must be the same */
+    ret = gst_structure_get_int (structure, "depth", &depth);
+    if (!ret || width != depth)
+      goto not_equal;
+
+    if (width == 16)
+      format = RESAMPLE_FORMAT_S16;
+    else if (width == 32)
+      format = RESAMPLE_FORMAT_S32;
+    else
+      goto wrong_depth;
+  }
   ret = gst_structure_get_int (structure, "rate", &myinrate);
   ret &= gst_structure_get_int (structure, "channels", &mychannels);
-  g_return_val_if_fail (ret, FALSE);
+  if (!ret)
+    goto no_in_rate_channels;
 
   structure = gst_caps_get_structure (outcaps, 0);
   ret = gst_structure_get_int (structure, "rate", &myoutrate);
-  g_return_val_if_fail (ret, FALSE);
+  if (!ret)
+    goto no_out_rate;
 
   if (channels)
     *channels = mychannels;
@@ -267,24 +333,53 @@ static gboolean
   if (outrate)
     *outrate = myoutrate;
 
+  resample_set_format (state, format);
   resample_set_n_channels (state, mychannels);
   resample_set_input_rate (state, myinrate);
   resample_set_output_rate (state, myoutrate);
 
   return TRUE;
+
+  /* ERRORS */
+no_width:
+  {
+    GST_DEBUG ("failed to get width from caps");
+    return FALSE;
+  }
+not_equal:
+  {
+    GST_DEBUG ("width %d and depth %d must be the same", width, depth);
+    return FALSE;
+  }
+wrong_depth:
+  {
+    GST_DEBUG ("unknown depth %d found", depth);
+    return FALSE;
+  }
+no_in_rate_channels:
+  {
+    GST_DEBUG ("could not get input rate and channels");
+    return FALSE;
+  }
+no_out_rate:
+  {
+    GST_DEBUG ("could not get output rate");
+    return FALSE;
+  }
 }
 
-gboolean
-    audioresample_transform_size (GstBaseTransform * base,
+static gboolean
+audioresample_transform_size (GstBaseTransform * base,
     GstPadDirection direction, GstCaps * caps, guint size, GstCaps * othercaps,
-    guint * othersize) {
+    guint * othersize)
+{
   GstAudioresample *audioresample = GST_AUDIORESAMPLE (base);
   ResampleState *state;
   GstCaps *srccaps, *sinkcaps;
   gboolean use_internal = FALSE;        /* whether we use the internal state */
   gboolean ret = TRUE;
 
-  GST_DEBUG_OBJECT (base, "asked to transform size %d in direction %s",
+  GST_LOG_OBJECT (base, "asked to transform size %d in direction %s",
       size, direction == GST_PAD_SINK ? "SINK" : "SRC");
   if (direction == GST_PAD_SINK) {
     sinkcaps = caps;
@@ -319,7 +414,7 @@ gboolean
 
   /* we make room for one extra sample, given that the resampling filter
    * can output an extra one for non-integral i_rate/o_rate */
-  GST_DEBUG_OBJECT (base, "transformed size %d to %d", size, *othersize);
+  GST_LOG_OBJECT (base, "transformed size %d to %d", size, *othersize);
 
   if (!use_internal) {
     resample_free (state);
@@ -328,9 +423,10 @@ gboolean
   return ret;
 }
 
-gboolean
-    audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
-    GstCaps * outcaps) {
+static gboolean
+audioresample_set_caps (GstBaseTransform * base, GstCaps * incaps,
+    GstCaps * outcaps)
+{
   gboolean ret;
   gint inrate, outrate;
   int channels;
@@ -353,16 +449,14 @@ gboolean
 
   /* save caps so we can short-circuit in the size_transform if the caps
    * are the same */
-  /* FIXME: clean them up in state change ? */
-  gst_caps_ref (incaps);
   gst_caps_replace (&audioresample->sinkcaps, incaps);
-  gst_caps_ref (outcaps);
   gst_caps_replace (&audioresample->srccaps, outcaps);
 
   return TRUE;
 }
 
-static gboolean audioresample_event (GstBaseTransform * base, GstEvent * event)
+static gboolean
+audioresample_event (GstBaseTransform * base, GstEvent * event)
 {
   GstAudioresample *audioresample;
 
@@ -397,8 +491,7 @@ static gboolean audioresample_event (GstBaseTransform * base, GstEvent * event)
 }
 
 static GstFlowReturn
-    audioresample_do_output (GstAudioresample * audioresample,
-    GstBuffer * outbuf)
+audioresample_do_output (GstAudioresample * audioresample, GstBuffer * outbuf)
 {
   int outsize;
   int outsamples;
@@ -407,8 +500,7 @@ static GstFlowReturn
   r = audioresample->resample;
 
   outsize = resample_get_output_size (r);
-  GST_DEBUG_OBJECT (audioresample, "audioresample can give me %d bytes",
-      outsize);
+  GST_LOG_OBJECT (audioresample, "audioresample can give me %d bytes", outsize);
 
   /* protect against mem corruption */
   if (outsize > GST_BUFFER_SIZE (outbuf)) {
@@ -455,8 +547,8 @@ static GstFlowReturn
   /* check for possible mem corruption */
   if (outsize > GST_BUFFER_SIZE (outbuf)) {
     /* this is an error that when it happens, would need fixing in the
-     * resample library; we told
-     * it we wanted only GST_BUFFER_SIZE (outbuf), and it gave us more ! */
+     * resample library; we told it we wanted only GST_BUFFER_SIZE (outbuf),
+     * and it gave us more ! */
     GST_WARNING_OBJECT (audioresample,
         "audioresample, you memory corrupting bastard. "
         "you gave me outsize %d while my buffer was size %d",
@@ -471,11 +563,51 @@ static GstFlowReturn
   }
   GST_BUFFER_SIZE (outbuf) = outsize;
 
+  if (G_UNLIKELY (audioresample->need_discont)) {
+    GST_DEBUG_OBJECT (audioresample,
+        "marking this buffer with the DISCONT flag");
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+    audioresample->need_discont = FALSE;
+  }
+
+  GST_LOG_OBJECT (audioresample, "transformed to buffer of %d bytes, ts %"
+      GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT ", offset %"
+      G_GINT64_FORMAT ", offset_end %" G_GINT64_FORMAT,
+      outsize, GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
+      GST_TIME_ARGS (GST_BUFFER_DURATION (outbuf)),
+      GST_BUFFER_OFFSET (outbuf), GST_BUFFER_OFFSET_END (outbuf));
+
+
   return GST_FLOW_OK;
 }
 
+static gboolean
+audioresample_check_discont (GstAudioresample * audioresample,
+    GstClockTime timestamp)
+{
+  if (timestamp != GST_CLOCK_TIME_NONE &&
+      audioresample->prev_ts != GST_CLOCK_TIME_NONE &&
+      audioresample->prev_duration != GST_CLOCK_TIME_NONE &&
+      timestamp != audioresample->prev_ts + audioresample->prev_duration) {
+    /* Potentially a discontinuous buffer. However, it turns out that many
+     * elements generate imperfect streams due to rounding errors, so we permit
+     * a small error (up to one sample) without triggering a filter 
+     * flush/restart (if triggered incorrectly, this will be audible) */
+    GstClockTimeDiff diff = timestamp -
+        (audioresample->prev_ts + audioresample->prev_duration);
+
+    if (ABS (diff) > GST_SECOND / audioresample->i_rate) {
+      GST_WARNING_OBJECT (audioresample,
+          "encountered timestamp discontinuity of %" G_GINT64_FORMAT, diff);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
 static GstFlowReturn
-    audioresample_transform (GstBaseTransform * base, GstBuffer * inbuf,
+audioresample_transform (GstBaseTransform * base, GstBuffer * inbuf,
     GstBuffer * outbuf)
 {
   GstAudioresample *audioresample;
@@ -491,7 +623,22 @@ static GstFlowReturn
   size = GST_BUFFER_SIZE (inbuf);
   timestamp = GST_BUFFER_TIMESTAMP (inbuf);
 
-  GST_DEBUG_OBJECT (audioresample, "got buffer of %ld bytes", size);
+  GST_LOG_OBJECT (audioresample, "transforming buffer of %ld bytes, ts %"
+      GST_TIME_FORMAT ", duration %" GST_TIME_FORMAT ", offset %"
+      G_GINT64_FORMAT ", offset_end %" G_GINT64_FORMAT,
+      size, GST_TIME_ARGS (timestamp),
+      GST_TIME_ARGS (GST_BUFFER_DURATION (inbuf)),
+      GST_BUFFER_OFFSET (inbuf), GST_BUFFER_OFFSET_END (inbuf));
+
+  /* check for timestamp discontinuities and flush/reset if needed */
+  if (G_UNLIKELY (audioresample_check_discont (audioresample, timestamp))) {
+    /* Flush internal samples */
+    audioresample_pushthrough (audioresample);
+    /* Inform downstream element about discontinuity */
+    audioresample->need_discont = TRUE;
+    /* We want to recalculate the offset */
+    audioresample->ts_offset = -1;
+  }
 
   if (audioresample->ts_offset == -1) {
     /* if we don't know the initial offset yet, calculate it based on the 
@@ -499,19 +646,21 @@ static GstFlowReturn
     if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
       GstClockTime stime;
 
-      /* offset used to calculate the timestamps. We use the sample offset for this
-       * to make it more accurate. We want the first buffer to have the same timestamp
-       * as the incomming timestamp. */
+      /* offset used to calculate the timestamps. We use the sample offset for
+       * this to make it more accurate. We want the first buffer to have the
+       * same timestamp as the incoming timestamp. */
       audioresample->next_ts = timestamp;
       audioresample->ts_offset =
           gst_util_uint64_scale_int (timestamp, r->o_rate, GST_SECOND);
-      /* offset used to set as the buffer offset, this offset is always relative
-       * to the stream time, note that timestamp is not... */
+      /* offset used to set as the buffer offset, this offset is always
+       * relative to the stream time, note that timestamp is not... */
       stime = (timestamp - base->segment.start) + base->segment.time;
       audioresample->offset =
           gst_util_uint64_scale_int (stime, r->o_rate, GST_SECOND);
     }
   }
+  audioresample->prev_ts = timestamp;
+  audioresample->prev_duration = GST_BUFFER_DURATION (inbuf);
 
   /* need to memdup, resample takes ownership. */
   datacopy = g_memdup (data, size);
@@ -522,7 +671,7 @@ static GstFlowReturn
 
 /* push remaining data in the buffers out */
 static GstFlowReturn
-    audioresample_pushthrough (GstAudioresample * audioresample)
+audioresample_pushthrough (GstAudioresample * audioresample)
 {
   int outsize;
   ResampleState *r;
@@ -533,57 +682,138 @@ static GstFlowReturn
   r = audioresample->resample;
 
   outsize = resample_get_output_size (r);
-  if (outsize == 0)
-    goto done;
-
-  outbuf = gst_buffer_new_and_alloc (outsize);
-
-  res = audioresample_do_output (audioresample, outbuf);
-  if (res != GST_FLOW_OK)
+  if (outsize == 0) {
+    GST_DEBUG_OBJECT (audioresample, "no internal buffers needing flush");
     goto done;
+  }
 
   trans = GST_BASE_TRANSFORM (audioresample);
 
+  res = gst_pad_alloc_buffer (trans->srcpad, GST_BUFFER_OFFSET_NONE, outsize,
+      GST_PAD_CAPS (trans->srcpad), &outbuf);
+  if (G_UNLIKELY (res != GST_FLOW_OK)) {
+    GST_WARNING_OBJECT (audioresample, "failed allocating buffer of %d bytes",
+        outsize);
+    goto done;
+  }
+
+  res = audioresample_do_output (audioresample, outbuf);
+  if (G_UNLIKELY (res != GST_FLOW_OK))
+    goto done;
+
   res = gst_pad_push (trans->srcpad, outbuf);
 
 done:
   return res;
 }
 
+static gboolean
+audioresample_query (GstPad * pad, GstQuery * query)
+{
+  GstAudioresample *audioresample =
+      GST_AUDIORESAMPLE (gst_pad_get_parent (pad));
+  GstBaseTransform *trans = GST_BASE_TRANSFORM (audioresample);
+  gboolean res = TRUE;
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_LATENCY:
+    {
+      GstClockTime min, max;
+      gboolean live;
+      guint64 latency;
+      GstPad *peer;
+      gint rate = audioresample->i_rate;
+      gint resampler_latency = audioresample->filter_length / 2;
+
+      if (gst_base_transform_is_passthrough (trans))
+        resampler_latency = 0;
+
+      if ((peer = gst_pad_get_peer (trans->sinkpad))) {
+        if ((res = gst_pad_query (peer, query))) {
+          gst_query_parse_latency (query, &live, &min, &max);
+
+          GST_DEBUG ("Peer latency: min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+          /* add our own latency */
+          if (rate != 0 && resampler_latency != 0)
+            latency =
+                gst_util_uint64_scale (resampler_latency, GST_SECOND, rate);
+          else
+            latency = 0;
+
+          GST_DEBUG ("Our latency: %" GST_TIME_FORMAT, GST_TIME_ARGS (latency));
+
+          min += latency;
+          if (max != GST_CLOCK_TIME_NONE)
+            max += latency;
+
+          GST_DEBUG ("Calculated total latency : min %"
+              GST_TIME_FORMAT " max %" GST_TIME_FORMAT,
+              GST_TIME_ARGS (min), GST_TIME_ARGS (max));
+
+          gst_query_set_latency (query, live, min, max);
+        }
+        gst_object_unref (peer);
+      }
+      break;
+    }
+    default:
+      res = gst_pad_query_default (pad, query);
+      break;
+  }
+  gst_object_unref (audioresample);
+  return res;
+}
+
+static const GstQueryType *
+audioresample_query_type (GstPad * pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_LATENCY,
+    0
+  };
+
+  return types;
+}
 
 static void
-    gst_audioresample_set_property (GObject * object, guint prop_id,
+gst_audioresample_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstAudioresample *audioresample;
 
-    g_return_if_fail (GST_IS_AUDIORESAMPLE (object));
-    audioresample = GST_AUDIORESAMPLE (object);
+  audioresample = GST_AUDIORESAMPLE (object);
 
   switch (prop_id) {
-    case ARG_FILTERLEN:
+    case PROP_FILTERLEN:
       audioresample->filter_length = g_value_get_int (value);
       GST_DEBUG_OBJECT (GST_ELEMENT (audioresample), "new filter length %d",
           audioresample->filter_length);
-      resample_set_filter_length (audioresample->resample,
-          audioresample->filter_length);
+      if (audioresample->resample) {
+        resample_set_filter_length (audioresample->resample,
+            audioresample->filter_length);
+        gst_element_post_message (GST_ELEMENT (audioresample),
+            gst_message_new_latency (GST_OBJECT (audioresample)));
+      }
       break;
-      default:G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
 static void
-    gst_audioresample_get_property (GObject * object, guint prop_id,
+gst_audioresample_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
   GstAudioresample *audioresample;
 
-  g_return_if_fail (GST_IS_AUDIORESAMPLE (object));
   audioresample = GST_AUDIORESAMPLE (object);
 
   switch (prop_id) {
-    case ARG_FILTERLEN:
+    case PROP_FILTERLEN:
       g_value_set_int (value, audioresample->filter_length);
       break;
     default:
@@ -593,7 +823,8 @@ static void
 }
 
 
-static gboolean plugin_init (GstPlugin * plugin)
+static gboolean
+plugin_init (GstPlugin * plugin)
 {
   resample_init ();
 
