@@ -1,5 +1,6 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ *               <2007> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -109,6 +110,9 @@ struct _GstPlayBaseBinClass {
   /* virtual fuctions */
   gboolean (*setup_output_pads) (GstPlayBaseBin *play_base_bin,
                                  GstPlayBaseGroup *group);
+
+  void     (*set_subtitles_visible) (GstPlayBaseBin *play_base_bin,
+                                     gboolean visible);
 };
 
 GType gst_play_base_bin_get_type (void);
