@@ -22,10 +22,16 @@
 
 #include <glib.h>
 
+enum {
+  VTS_YUV,
+  VTS_RGB,
+  VTS_BAYER
+};
 
 struct vts_color_struct {
         guint8 Y, U, V;
         guint8 R, G, B;
+	guint8 A;
 };
 
 typedef struct paintinfo_struct paintinfo;
@@ -34,28 +40,30 @@ struct paintinfo_struct
   unsigned char *dest;          /* pointer to first byte of video data */
   unsigned char *yp, *up, *vp;  /* pointers to first byte of each component
                                  * for both packed/planar YUV and RGB */
+  unsigned char *ap;            /* pointer to first byte of alpha component */
   unsigned char *endptr;        /* pointer to byte beyond last video data */
   int ystride;
   int ustride;
   int vstride;
   int width;
   int height;
-  struct vts_color_struct *color;
+  const struct vts_color_struct *color;
   void (*paint_hline) (paintinfo * p, int x, int y, int w);
 };
 
 struct fourcc_list_struct
 {
+  int type;
   char *fourcc;
   char *name;
   int bitspp;
   void (*paint_setup) (paintinfo * p, unsigned char *dest);
   void (*paint_hline) (paintinfo * p, int x, int y, int w);
-  int ext_caps;
   int depth;
   unsigned int red_mask;
   unsigned int green_mask;
   unsigned int blue_mask;
+  unsigned int alpha_mask;
 };
 
 struct fourcc_list_struct *
@@ -67,11 +75,29 @@ struct fourcc_list_struct *
 GstStructure *
         paint_get_structure             (struct fourcc_list_struct *format);
 int     gst_video_test_src_get_size     (GstVideoTestSrc * v, int w, int h);
-void    gst_video_test_src_smpte                (GstVideoTestSrc * v,
+void    gst_video_test_src_smpte        (GstVideoTestSrc * v,
                                          unsigned char *dest, int w, int h);
 void    gst_video_test_src_snow         (GstVideoTestSrc * v,
                                          unsigned char *dest, int w, int h);
-void    gst_video_test_src_black                (GstVideoTestSrc * v,
+void    gst_video_test_src_black        (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_white        (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_red          (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_green        (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_blue         (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_checkers1    (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_checkers2    (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_checkers4    (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_checkers8    (GstVideoTestSrc * v,
+                                         unsigned char *dest, int w, int h);
+void    gst_video_test_src_circular     (GstVideoTestSrc * v,
                                          unsigned char *dest, int w, int h);
 
 extern struct fourcc_list_struct fourcc_list[];
