@@ -50,19 +50,33 @@
 #include <gst/audio/audio.h>
 #include <gst/interfaces/mixer.h>
 #include <gst/controller/gstcontroller.h>
+#include <gst/audio/audio.h>
+#include <gst/audio/gstaudiofilter.h>
 #include <liboil/liboil.h>
 
 #include "gstvolume.h"
 
 /* some defines for audio processing */
 /* the volume factor is a range from 0.0 to (arbitrary) VOLUME_MAX_DOUBLE = 10.0
- * we map 1.0 to VOLUME_UNITY_INT16
+ * we map 1.0 to VOLUME_UNITY_INT*
  */
-#define VOLUME_UNITY_INT16           8192       /* internal int for unity 2^13 */
+#define VOLUME_UNITY_INT8            32 /* internal int for unity 2^(8-3) */
+#define VOLUME_UNITY_INT8_BIT_SHIFT  5  /* number of bits to shift for unity */
+#define VOLUME_UNITY_INT16           8192       /* internal int for unity 2^(16-3) */
 #define VOLUME_UNITY_INT16_BIT_SHIFT 13 /* number of bits to shift for unity */
+#define VOLUME_UNITY_INT24           2097152    /* internal int for unity 2^(24-3) */
+#define VOLUME_UNITY_INT24_BIT_SHIFT 21 /* number of bits to shift for unity */
+#define VOLUME_UNITY_INT32           134217728  /* internal int for unity 2^(32-5) */
+#define VOLUME_UNITY_INT32_BIT_SHIFT 27
 #define VOLUME_MAX_DOUBLE            10.0
-#define VOLUME_MAX_INT16             32767
-#define VOLUME_MIN_INT16             -32768
+#define VOLUME_MAX_INT8              G_MAXINT8
+#define VOLUME_MIN_INT8              G_MININT8
+#define VOLUME_MAX_INT16             G_MAXINT16
+#define VOLUME_MIN_INT16             G_MININT16
+#define VOLUME_MAX_INT24             8388607
+#define VOLUME_MIN_INT24             -8388608
+#define VOLUME_MAX_INT32             G_MAXINT32
+#define VOLUME_MIN_INT32             G_MININT32
 
 /* number of steps we use for the mixer interface to go from 0.0 to 1.0 */
 # define VOLUME_STEPS           100
@@ -90,37 +104,40 @@ enum
   PROP_VOLUME
 };
 
-static GstStaticPadTemplate volume_sink_template =
-    GST_STATIC_PAD_TEMPLATE ("sink",
-    GST_PAD_SINK,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-float, "
-        "rate = (int) [ 1, MAX ], "
-        "channels = (int) [ 1, MAX ], "
-        "endianness = (int) BYTE_ORDER, "
-        "width = (int) {32, 64}; "
-        "audio/x-raw-int, "
-        "channels = (int) [ 1, MAX ], "
-        "rate = (int) [ 1,  MAX ], "
-        "endianness = (int) BYTE_ORDER, "
-        "width = (int) 16, " "depth = (int) 16, " "signed = (bool) TRUE")
-    );
-
-static GstStaticPadTemplate volume_src_template = GST_STATIC_PAD_TEMPLATE
-    ("src",
-    GST_PAD_SRC,
-    GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-float, "
-        "rate = (int) [ 1, MAX ], "
-        "channels = (int) [ 1, MAX ], "
-        "endianness = (int) BYTE_ORDER, "
-        "width = (int) {32, 64}; "
-        "audio/x-raw-int, "
-        "channels = (int) [ 1, MAX ], "
-        "rate = (int) [ 1,  MAX ], "
-        "endianness = (int) BYTE_ORDER, "
-        "width = (int) 16, " "depth = (int) 16, " "signed = (bool) TRUE")
-    );
+#define ALLOWED_CAPS \
+        "audio/x-raw-float, " \
+        "rate = (int) [ 1, MAX ], " \
+        "channels = (int) [ 1, MAX ], " \
+        "endianness = (int) BYTE_ORDER, " \
+        "width = (int) {32, 64}; " \
+        "audio/x-raw-int, " \
+        "channels = (int) [ 1, MAX ], " \
+        "rate = (int) [ 1,  MAX ], " \
+        "endianness = (int) BYTE_ORDER, " \
+        "width = (int) 8, " \
+        "depth = (int) 8, " \
+        "signed = (bool) TRUE; " \
+        "audio/x-raw-int, " \
+        "channels = (int) [ 1, MAX ], " \
+        "rate = (int) [ 1,  MAX ], " \
+        "endianness = (int) BYTE_ORDER, " \
+        "width = (int) 16, " \
+        "depth = (int) 16, " \
+        "signed = (bool) TRUE; " \
+        "audio/x-raw-int, " \
+        "channels = (int) [ 1, MAX ], " \
+        "rate = (int) [ 1,  MAX ], " \
+        "endianness = (int) BYTE_ORDER, " \
+        "width = (int) 24, " \
+        "depth = (int) 24, " \
+        "signed = (bool) TRUE; " \
+        "audio/x-raw-int, " \
+        "channels = (int) [ 1, MAX ], " \
+        "rate = (int) [ 1,  MAX ], " \
+        "endianness = (int) BYTE_ORDER, " \
+        "width = (int) 32, " \
+	"depth = (int) 32, " \
+	"signed = (bool) TRUE"
 
 static void gst_volume_interface_init (GstImplementsInterfaceClass * klass);
 static void gst_volume_mixer_init (GstMixerClass * iface);
@@ -143,8 +160,8 @@ static void gst_volume_mixer_init (GstMixerClass * iface);
     g_type_add_interface_static (type, GST_TYPE_MIXER, &volmixer_info); \
   }
 
-GST_BOILERPLATE_FULL (GstVolume, gst_volume, GstBaseTransform,
-    GST_TYPE_BASE_TRANSFORM, _init_interfaces);
+GST_BOILERPLATE_FULL (GstVolume, gst_volume, GstAudioFilter,
+    GST_TYPE_AUDIO_FILTER, _init_interfaces);
 
 static void volume_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -155,37 +172,84 @@ static void volume_update_mute (const GValue * value, gpointer data);
 
 static GstFlowReturn volume_transform_ip (GstBaseTransform * base,
     GstBuffer * outbuf);
-static gboolean volume_set_caps (GstBaseTransform * base, GstCaps * incaps,
-    GstCaps * outcaps);
+static gboolean volume_setup (GstAudioFilter * filter,
+    GstRingBufferSpec * format);
 
 static void volume_process_double (GstVolume * this, gpointer bytes,
     guint n_bytes);
 static void volume_process_float (GstVolume * this, gpointer bytes,
     guint n_bytes);
+static void volume_process_int32 (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int32_clamp (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int24 (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int24_clamp (GstVolume * this, gpointer bytes,
+    guint n_bytes);
 static void volume_process_int16 (GstVolume * this, gpointer bytes,
     guint n_bytes);
 static void volume_process_int16_clamp (GstVolume * this, gpointer bytes,
     guint n_bytes);
+static void volume_process_int8 (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+static void volume_process_int8_clamp (GstVolume * this, gpointer bytes,
+    guint n_bytes);
+
 
 /* helper functions */
 
-static void
+static gboolean
 volume_choose_func (GstVolume * this)
 {
   this->process = NULL;
-  switch (this->format) {
-    case GST_VOLUME_FORMAT_INT:
-      /* FIXME: should we also support gint8, gint32? */
-      /* only clamp if the gain is greater than 1.0
-       * FIXME: real_vol_i can change while processing the buffer!
-       */
-      if (this->real_vol_i > VOLUME_UNITY_INT16)
-        this->process = volume_process_int16_clamp;
-      else
-        this->process = volume_process_int16;
+
+  if (GST_AUDIO_FILTER (this)->format.caps == NULL)
+    return FALSE;
+
+  switch (GST_AUDIO_FILTER (this)->format.type) {
+    case GST_BUFTYPE_LINEAR:
+      switch (GST_AUDIO_FILTER (this)->format.width) {
+        case 32:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i32 > VOLUME_UNITY_INT32)
+            this->process = volume_process_int32_clamp;
+          else
+            this->process = volume_process_int32;
+          break;
+        case 24:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i24 > VOLUME_UNITY_INT24)
+            this->process = volume_process_int24_clamp;
+          else
+            this->process = volume_process_int24;
+          break;
+        case 16:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i16 > VOLUME_UNITY_INT16)
+            this->process = volume_process_int16_clamp;
+          else
+            this->process = volume_process_int16;
+          break;
+        case 8:
+          /* only clamp if the gain is greater than 1.0
+           * FIXME: real_vol_i can change while processing the buffer!
+           */
+          if (this->real_vol_i16 > VOLUME_UNITY_INT8)
+            this->process = volume_process_int8_clamp;
+          else
+            this->process = volume_process_int8;
+          break;
+      }
       break;
-    case GST_VOLUME_FORMAT_FLOAT:
-      switch (this->width) {
+    case GST_BUFTYPE_FLOAT:
+      switch (GST_AUDIO_FILTER (this)->format.width) {
         case 32:
           this->process = volume_process_float;
           break;
@@ -197,6 +261,8 @@ volume_choose_func (GstVolume * this)
     default:
       break;
   }
+
+  return (this->process != NULL);
 }
 
 static void
@@ -206,12 +272,18 @@ volume_update_real_volume (GstVolume * this)
 
   if (this->mute) {
     this->real_vol_f = 0.0;
-    this->real_vol_i = 0;
+    this->real_vol_i8 = this->real_vol_i16 = this->real_vol_i24 =
+        this->real_vol_i32 = 0;
   } else {
     this->real_vol_f = this->volume_f;
-    this->real_vol_i = this->volume_i;
-    passthrough = (this->volume_i == VOLUME_UNITY_INT16);
+    this->real_vol_i8 = this->volume_i8;
+    this->real_vol_i16 = this->volume_i16;
+    this->real_vol_i24 = this->volume_i24;
+    this->real_vol_i32 = this->volume_i32;
+    passthrough = (this->volume_i16 == VOLUME_UNITY_INT16);
   }
+  if (this->real_vol_f != 0.0)
+    this->silent_buffer = FALSE;
   volume_choose_func (this);
   gst_base_transform_set_passthrough (GST_BASE_TRANSFORM (this), passthrough);
 }
@@ -251,7 +323,10 @@ gst_volume_set_volume (GstMixer * mixer, GstMixerTrack * track, gint * volumes)
   g_return_if_fail (GST_IS_VOLUME (this));
 
   this->volume_f = (gfloat) volumes[0] / VOLUME_STEPS;
-  this->volume_i = this->volume_f * VOLUME_UNITY_INT16;
+  this->volume_i32 = this->volume_f * VOLUME_UNITY_INT32;
+  this->volume_i24 = this->volume_f * VOLUME_UNITY_INT24;
+  this->volume_i16 = this->volume_f * VOLUME_UNITY_INT16;
+  this->volume_i8 = this->volume_f * VOLUME_UNITY_INT8;
 
   volume_update_real_volume (this);
 }
@@ -313,12 +388,14 @@ static void
 gst_volume_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstAudioFilterClass *filter_class = GST_AUDIO_FILTER_CLASS (g_class);
+  GstCaps *caps;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&volume_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&volume_sink_template));
   gst_element_class_set_details (element_class, &volume_details);
+
+  caps = gst_caps_from_string (ALLOWED_CAPS);
+  gst_audio_filter_class_add_pad_templates (filter_class, caps);
+  gst_caps_unref (caps);
 }
 
 static void
@@ -326,9 +403,11 @@ gst_volume_class_init (GstVolumeClass * klass)
 {
   GObjectClass *gobject_class;
   GstBaseTransformClass *trans_class;
+  GstAudioFilterClass *filter_class;
 
   gobject_class = (GObjectClass *) klass;
   trans_class = (GstBaseTransformClass *) klass;
+  filter_class = (GstAudioFilterClass *) (klass);
 
   gobject_class->set_property = volume_set_property;
   gobject_class->get_property = volume_get_property;
@@ -346,7 +425,7 @@ gst_volume_class_init (GstVolumeClass * klass)
   GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "volume", 0, "Volume gain");
 
   trans_class->transform_ip = GST_DEBUG_FUNCPTR (volume_transform_ip);
-  trans_class->set_caps = GST_DEBUG_FUNCPTR (volume_set_caps);
+  filter_class->setup = GST_DEBUG_FUNCPTR (volume_setup);
 }
 
 static void
@@ -355,12 +434,12 @@ gst_volume_init (GstVolume * this, GstVolumeClass * g_class)
   GstMixerTrack *track = NULL;
 
   this->mute = FALSE;
-  this->volume_i = VOLUME_UNITY_INT16;
-  this->volume_f = 1.0;
-  this->real_vol_i = VOLUME_UNITY_INT16;
-  this->real_vol_f = 1.0;
+  this->volume_i8 = this->real_vol_i8 = VOLUME_UNITY_INT8;
+  this->volume_i16 = this->real_vol_i16 = VOLUME_UNITY_INT16;
+  this->volume_i24 = this->real_vol_i24 = VOLUME_UNITY_INT24;
+  this->volume_i32 = this->real_vol_i32 = VOLUME_UNITY_INT32;
+  this->volume_f = this->real_vol_f = 1.0;
   this->tracklist = NULL;
-  this->format = GST_VOLUME_FORMAT_NONE;
 
   track = g_object_new (GST_TYPE_MIXER_TRACK, NULL);
 
@@ -372,20 +451,20 @@ gst_volume_init (GstVolume * this, GstVolumeClass * g_class)
     track->flags = GST_MIXER_TRACK_SOFTWARE;
     this->tracklist = g_list_append (this->tracklist, track);
   }
+
+  gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (this), TRUE);
 }
 
 /* NOTE: although it might be tempting to have volume_process_mute() which uses
  *       memset(bytes, 0, nbytes) for the vol=0 case, this has the downside that
- *       unmuting would unly take place after processing a buffer.
+ *       unmuting would only take place after processing a buffer.
  */
 
 static void
 volume_process_double (GstVolume * this, gpointer bytes, guint n_bytes)
 {
   gdouble *data = (gdouble *) bytes;
-  guint i, num_samples;
-
-  num_samples = n_bytes / sizeof (gdouble);
+  guint i, num_samples = n_bytes / sizeof (gdouble);
 
   for (i = 0; i < num_samples; i++) {
     *data++ *= this->real_vol_f;
@@ -396,37 +475,130 @@ static void
 volume_process_float (GstVolume * this, gpointer bytes, guint n_bytes)
 {
   gfloat *data = (gfloat *) bytes;
-  guint i, num_samples;
+  guint num_samples = n_bytes / sizeof (gfloat);
 
-  num_samples = n_bytes / sizeof (gfloat);
+  /*
+     guint i;
+     for (i = 0; i < num_samples; i++) {
+     *data++ *= this->real_vol_f;
+     }
+   */
+  /* time gst-launch 2>/dev/null audiotestsrc wave=7 num-buffers=10000 ! audio/x-raw-float ! volume volume=1.5 ! fakesink
+   * goes from 0m0.850s -> 0m0.717s with liboil
+   */
+  oil_scalarmultiply_f32_ns (data, data, &this->real_vol_f, num_samples);
+}
+
+static void
+volume_process_int32 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint *data = (gint *) bytes;
+  guint i, num_samples;
+  gint64 val;
+
+  num_samples = n_bytes / sizeof (gint);
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint64) * data;
+    val = (((gint64) this->real_vol_i32 * val) >> VOLUME_UNITY_INT32_BIT_SHIFT);
+    *data++ = (gint32) val;
+  }
+}
+
+static void
+volume_process_int32_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint *data = (gint *) bytes;
+  guint i, num_samples;
+  gint64 val;
+
+  num_samples = n_bytes / sizeof (gint);
 
   for (i = 0; i < num_samples; i++) {
-    *data++ *= this->real_vol_f;
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint64) * data;
+    val = (((gint64) this->real_vol_i32 * val) >> VOLUME_UNITY_INT32_BIT_SHIFT);
+    *data++ = (gint32) CLAMP (val, VOLUME_MIN_INT32, VOLUME_MAX_INT32);
+  }
+}
+
+#if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
+#define get_unaligned_i24(_x) ( (((guint8*)_x)[0]) | ((((guint8*)_x)[1]) << 8) | ((((gint8*)_x)[2]) << 16) )
+#define write_unaligned_u24(_x,samp) do { *(_x)++ = samp & 0xFF; *(_x)++ = (samp >> 8) & 0xFF; *(_x)++ = (samp >> 16) & 0xFF; } while (0)
+#else /* BIG ENDIAN */
+#define get_unaligned_i24(_x) ( (((guint8*)_x)[2]) | ((((guint8*)_x)[1]) << 8) | ((((gint8*)_x)[0]) << 16) )
+#define write_unaligned_u24(_x,samp) do { *(_x)++ = (samp >> 16) & 0xFF; *(_x)++ = (samp >> 8) & 0xFF; *(_x)++ = samp & 0xFF; } while (0)
+#endif
+
+static void
+volume_process_int24 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;        /* treat the data as a byte stream */
+  guint i, num_samples;
+  guint32 samp;
+  gint64 val;
+
+  num_samples = n_bytes / (sizeof (gint8) * 3);
+  for (i = 0; i < num_samples; i++) {
+    samp = get_unaligned_i24 (data);
+
+    val = (gint32) samp;
+    val = (((gint64) this->real_vol_i24 * val) >> VOLUME_UNITY_INT24_BIT_SHIFT);
+    samp = (guint32) val;
+
+    /* write the value back into the stream */
+    write_unaligned_u24 (data, samp);
+  }
+}
+
+static void
+volume_process_int24_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;        /* treat the data as a byte stream */
+  guint i, num_samples;
+  guint32 samp;
+  gint64 val;
+
+  num_samples = n_bytes / (sizeof (gint8) * 3);
+  for (i = 0; i < num_samples; i++) {
+    samp = get_unaligned_i24 (data);
+
+    val = (gint32) samp;
+    val = (((gint64) this->real_vol_i24 * val) >> VOLUME_UNITY_INT24_BIT_SHIFT);
+    samp = (guint32) CLAMP (val, VOLUME_MIN_INT24, VOLUME_MAX_INT24);
+
+    /* write the value back into the stream */
+    write_unaligned_u24 (data, samp);
   }
-  /* FIXME: seems to be slower than above!
-     oil_scalarmultiply_f32_ns (data, data, &this->real_vol_f, num_samples);
-   */
 }
 
 static void
 volume_process_int16 (GstVolume * this, gpointer bytes, guint n_bytes)
 {
   gint16 *data = (gint16 *) bytes;
-  guint i, num_samples;
+  guint num_samples = n_bytes / sizeof (gint16);
+
+#if 1
+  guint i;
   gint val;
 
-  num_samples = n_bytes / sizeof (gint16);
-
-  /* FIXME: need... liboil...
-   * oil_scalarmultiply_s16_ns ?
-   * https://bugs.freedesktop.org/show_bug.cgi?id=7060
-   */
   for (i = 0; i < num_samples; i++) {
     /* we use bitshifting instead of dividing by UNITY_INT for speed */
     val = (gint) * data;
     *data++ =
-        (gint16) ((this->real_vol_i * val) >> VOLUME_UNITY_INT16_BIT_SHIFT);
+        (gint16) ((this->real_vol_i16 * val) >> VOLUME_UNITY_INT16_BIT_SHIFT);
   }
+#else
+  /* FIXME: need oil_scalarmultiply_s16_ns ?
+   * https://bugs.freedesktop.org/show_bug.cgi?id=7060
+   * code below
+   * - crashes :/
+   * - real_vol_i is scaled by VOLUME_UNITY_INT16 and needs the bitshift
+   * time gst-launch 2>/dev/null audiotestsrc wave=7 num-buffers=100 ! volume volume=1.5 ! fakesink
+   */
+  oil_scalarmult_s16 (data, 0, data, 0,
+      ((gint16 *) (void *) (&this->real_vol_i)), num_samples);
+#endif
 }
 
 static void
@@ -438,57 +610,67 @@ volume_process_int16_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
 
   num_samples = n_bytes / sizeof (gint16);
 
-  /* FIXME: need... liboil...
-   * oil_scalarmultiply_s16_ns ?
+  /* FIXME: oil_scalarmultiply_s16_ns ?
    * https://bugs.freedesktop.org/show_bug.cgi?id=7060
    */
   for (i = 0; i < num_samples; i++) {
     /* we use bitshifting instead of dividing by UNITY_INT for speed */
     val = (gint) * data;
     *data++ =
-        (gint16) CLAMP ((this->real_vol_i *
+        (gint16) CLAMP ((this->real_vol_i16 *
             val) >> VOLUME_UNITY_INT16_BIT_SHIFT, VOLUME_MIN_INT16,
         VOLUME_MAX_INT16);
   }
 }
 
+static void
+volume_process_int8 (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;
+  guint num_samples = n_bytes / sizeof (gint8);
+  guint i;
+  gint val;
+
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint) * data;
+    *data++ =
+        (gint8) ((this->real_vol_i8 * val) >> VOLUME_UNITY_INT8_BIT_SHIFT);
+  }
+}
+
+static void
+volume_process_int8_clamp (GstVolume * this, gpointer bytes, guint n_bytes)
+{
+  gint8 *data = (gint8 *) bytes;
+  guint i, num_samples;
+  gint val;
+
+  num_samples = n_bytes / sizeof (gint8);
+
+  for (i = 0; i < num_samples; i++) {
+    /* we use bitshifting instead of dividing by UNITY_INT for speed */
+    val = (gint) * data;
+    *data++ =
+        (gint8) CLAMP ((this->real_vol_i8 *
+            val) >> VOLUME_UNITY_INT8_BIT_SHIFT, VOLUME_MIN_INT8,
+        VOLUME_MAX_INT8);
+  }
+}
+
 /* GstBaseTransform vmethod implementations */
 
 /* get notified of caps and plug in the correct process function */
 static gboolean
-volume_set_caps (GstBaseTransform * base, GstCaps * incaps, GstCaps * outcaps)
+volume_setup (GstAudioFilter * filter, GstRingBufferSpec * format)
 {
-  GstVolume *this = GST_VOLUME (base);
-  const gchar *mimetype;
-  GstStructure *structure;
+  GstVolume *this = GST_VOLUME (filter);
 
-  GST_DEBUG_OBJECT (this,
-      "set_caps: in %" GST_PTR_FORMAT " out %" GST_PTR_FORMAT, incaps, outcaps);
-
-  structure = gst_caps_get_structure (incaps, 0);
-  gst_structure_get_int (structure, "width", &this->width);
-  mimetype = gst_structure_get_name (structure);
-
-  /* based on mimetype, choose the correct volume_process format */
-  if (strcmp (mimetype, "audio/x-raw-int") == 0) {
-    this->format = GST_VOLUME_FORMAT_INT;
-    GST_DEBUG_OBJECT (this, "use int: %u", this->width);
-  } else if (strcmp (mimetype, "audio/x-raw-float") == 0) {
-    this->format = GST_VOLUME_FORMAT_FLOAT;
-    GST_DEBUG_OBJECT (this, "use float: %u", this->width);
+  if (volume_choose_func (this)) {
+    return TRUE;
   } else {
-    this->process = NULL;
-    goto invalid_format;
-  }
-  volume_choose_func (this);
-
-  return TRUE;
-
-  /* ERRORS */
-invalid_format:
-  {
     GST_ELEMENT_ERROR (this, CORE, NEGOTIATION,
-        ("Invalid incoming caps: %" GST_PTR_FORMAT, incaps), (NULL));
+        ("Invalid incoming format"), (NULL));
     return FALSE;
   }
 }
@@ -503,10 +685,6 @@ volume_transform_ip (GstBaseTransform * base, GstBuffer * outbuf)
   GstVolume *this = GST_VOLUME (base);
   GstClockTime timestamp;
 
-  /* don't process data in passthrough-mode */
-  if (gst_base_transform_is_passthrough (base))
-    return GST_FLOW_OK;
-
   /* FIXME: if controllers are bound, subdivide GST_BUFFER_SIZE into small
    * chunks for smooth fades, what is small? 1/10th sec.
    */
@@ -520,8 +698,20 @@ volume_transform_ip (GstBaseTransform * base, GstBuffer * outbuf)
   if (GST_CLOCK_TIME_IS_VALID (timestamp))
     gst_object_sync_values (G_OBJECT (this), timestamp);
 
+  /* don't process data in passthrough-mode */
+  if (gst_base_transform_is_passthrough (base) ||
+      GST_BUFFER_FLAG_IS_SET (outbuf, GST_BUFFER_FLAG_GAP))
+    return GST_FLOW_OK;
+
+  if (this->real_vol_f == 0.0)
+    this->silent_buffer = TRUE;
+
   this->process (this, GST_BUFFER_DATA (outbuf), GST_BUFFER_SIZE (outbuf));
 
+  if (this->silent_buffer)
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_GAP);
+  this->silent_buffer = FALSE;
+
   return GST_FLOW_OK;
 }
 
@@ -549,7 +739,10 @@ volume_update_volume (const GValue * value, gpointer data)
   g_return_if_fail (GST_IS_VOLUME (this));
 
   this->volume_f = g_value_get_double (value);
-  this->volume_i = this->volume_f * VOLUME_UNITY_INT16;
+  this->volume_i8 = this->volume_f * VOLUME_UNITY_INT8;
+  this->volume_i16 = this->volume_f * VOLUME_UNITY_INT16;
+  this->volume_i24 = this->volume_f * VOLUME_UNITY_INT24;
+  this->volume_i32 = this->volume_f * VOLUME_UNITY_INT32;
 
   volume_update_real_volume (this);
 }
@@ -595,7 +788,7 @@ volume_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  /*oil_init (); */
+  oil_init ();
 
   /* initialize gst controller library */
   gst_controller_init (NULL, NULL);
